@@ -17,17 +17,23 @@
 #  KIND, either express or implied.  See the License for the
 #  specific language governing permissions and limitations
 #  under the License.
-import unittest
+import sys
+if sys.version_info[:2] < (2, 7):
+    import unittest2 as unittest
+else:
+    import unittest
 
-from urllib import urlencode, unquote
+from urllib import urlencode, unquote, unquote_plus
+
+from bhsearch import web_ui
 from bhsearch.api import ASC, DESC, SortInstruction
-
 from bhsearch.tests.base import BaseBloodhoundSearchTest
-from bhsearch.web_ui import RequestParameters
+from bhsearch.web_ui import RequestParameters, BloodhoundSearchModule
 from bhsearch.whoosh_backend import WhooshBackend
 
 from trac.test import Mock, MockPerm
 from trac.core import TracError
+from trac.search.web_ui import SearchModule as TracSearchModule
 from trac.util.datefmt import FixedOffset
 from trac.util import format_datetime
 from trac.web import Href, arg_list_to_args, parse_arg_list, RequestDone
@@ -46,18 +52,27 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         whoosh_backend = WhooshBackend(self.env)
         whoosh_backend.recreate_index()
 
-        self.req = Mock(
-            perm=MockPerm(),
-            chrome={'logo': {}},
-            href=Href("/main"),
-            base_path=BASE_PATH,
-            args=arg_list_to_args([]),
-            redirect=self.redirect
-        )
-
+        self.req.redirect = self.redirect
         self.redirect_url = None
         self.redirect_permanent = None
 
+        self.old_product_environment = web_ui.ProductEnvironment
+        self._inject_mocked_product_environment()
+
+    def _inject_mocked_product_environment(self):
+        class MockProductEnvironment(object):
+            def __init__(self, env, product):
+                # pylint: disable=unused-argument
+                self.product = product
+
+            def href(self, *args):
+                return ('/main/products/%s/' % self.product) + '/'.join(args)
+
+        web_ui.ProductEnvironment = MockProductEnvironment
+
+    def tearDown(self):
+        web_ui.ProductEnvironment = self.old_product_environment
+
     def redirect(self, url, permanent=False):
         self.redirect_url = url
         self.redirect_permanent = permanent
@@ -91,7 +106,8 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         #assert
         self.assertEqual(1, len(result_items))
         result_datetime = result_items[0]["date"]
-        print "Ticket time: %s, Returned time: %s" % (
+        self.env.log.debug(
+            "Ticket time: %s, Returned time: %s",
             ticket_time,
             result_datetime)
         self.assertEqual(format_datetime(ticket_time), result_items[0]["date"])
@@ -101,15 +117,17 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         ticket = self.insert_ticket("bla")
         ticket_time = ticket.time_changed
         #act
-        self.req.tz = FixedOffset(60, 'GMT +1:00')
+        tzinfo = FixedOffset(60, 'GMT +1:00')
+        self.req.tz = tzinfo
         self.req.args[RequestParameters.QUERY] = "*:*"
         data = self.process_request()
         result_items = data["results"].items
         #asset
         self.assertEqual(1, len(result_items))
-        expected_datetime = format_datetime(ticket_time)
+        expected_datetime = format_datetime(ticket_time, tzinfo=tzinfo)
         result_datetime = result_items[0]["date"]
-        print "Ticket time: %s, Formatted time: %s ,Returned time: %s" % (
+        self.env.log.debug(
+            "Ticket time: %s, Formatted time: %s ,Returned time: %s",
             ticket_time, expected_datetime,result_datetime)
         self.assertEqual(expected_datetime, result_datetime)
 
@@ -121,6 +139,15 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         self.assertEqual(1, len(docs))
         self.assertEqual("/main/ticket/1", docs[0]["href"])
 
+    def test_product_ticket_href(self):
+        with self.product('xxx'):
+            self._insert_tickets(1)
+        self.req.args[RequestParameters.QUERY] = "*:*"
+        data = self.process_request()
+        docs = data["results"].items
+        self.assertEqual(1, len(docs))
+        self.assertEqual("/main/products/xxx/ticket/1", docs[0]["href"])
+
     def test_page_href(self):
         self._insert_tickets(DEFAULT_DOCS_PER_PAGE+1)
         self.req.args[RequestParameters.QUERY] = "*:*"
@@ -141,19 +168,7 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         data = self.process_request()
         #assert
         extra_search_options = dict(data["extra_search_fields"])
-        self.assertEquals("ticket", extra_search_options['type'])
-
-        resource_types = data["types"]
-
-        all = resource_types[0]
-        self._assertResourceType(all, "All", False)
-        self.assertNotIn("type", all["href"])
-
-        ticket = resource_types[1]
-        self._assertResourceType(ticket, "Ticket", True, "type=ticket")
-
-        wiki = resource_types[2]
-        self._assertResourceType(wiki, "Wiki", False, "type=wiki")
+        self.assertEqual("ticket", extra_search_options['type'])
 
     def test_type_parameter_in_links(self):
         self._insert_tickets(12)
@@ -179,51 +194,6 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         for page in results.shown_pages:
             self.assertIn('type=ticket', page["href"])
 
-    def test_type_grouping(self):
-        self.req.args[RequestParameters.QUERY] = "*:*"
-        data = self.process_request()
-        resource_types =  data["types"]
-
-        all = resource_types[0]
-        self._assertResourceType(all, "All", True)
-        self.assertNotIn("type", all["href"])
-
-        ticket = resource_types[1]
-        self._assertResourceType(ticket, "Ticket", False, "type=ticket")
-
-        wiki = resource_types[2]
-        self._assertResourceType(wiki, "Wiki", False, "type=wiki")
-
-
-    def test_that_there_are_no_page_parameters_for_other_types(self):
-        #arrange
-        self._insert_tickets(12)
-        #act
-        self.req.args[RequestParameters.QUERY] = "*"
-        self.req.args[RequestParameters.PAGELEN] = "4"
-        self.req.args[RequestParameters.PAGE] = "2"
-        data = self.process_request()
-        #assert
-        resource_types =  data["types"]
-
-        all = resource_types[0]
-        self.assertNotIn("page=2", all["href"])
-
-        ticket = resource_types[1]
-        self.assertNotIn("page=", ticket["href"])
-
-    def test_that_there_are_filters_in_type_links(self):
-        #arrange
-#        self._insert_tickets(2)
-        #act
-        self.req.args[RequestParameters.QUERY] = "*"
-        self.req.args[RequestParameters.TYPE] = "ticket"
-        self.req.args[RequestParameters.FILTER_QUERY] = "status:new"
-        data = self.process_request()
-        #assert
-        for type in data["types"]:
-            self.assertNotIn("fq=", type["href"])
-
     def test_that_type_facet_is_in_default_search(self):
         #arrange
         self._insert_tickets(2)
@@ -242,7 +212,7 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         self.req.args[RequestParameters.QUERY] = "*"
         data = self.process_request()
         #assert
-        facet_counts =  data["facet_counts"]
+        facet_counts = dict(data["facet_counts"])
         status_counts = facet_counts["status"]
         self.assertEquals(1, status_counts["new"]["count"])
         self.assertEquals(1, status_counts["closed"]["count"])
@@ -256,7 +226,7 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         self.req.args[RequestParameters.QUERY] = "*"
         data = self.process_request()
         #assert
-        facet_counts =  data["facet_counts"]
+        facet_counts = dict(data["facet_counts"])
         status_counts = facet_counts["status"]
         self.assertEquals(1, status_counts["new"]["count"])
         self.assertIn("fq=status%3A%22new%22", status_counts["new"]["href"])
@@ -270,7 +240,7 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         self.req.args[RequestParameters.QUERY] = "*"
         data = self.process_request()
         #assert
-        facet_counts =  data["facet_counts"]
+        facet_counts = dict(data["facet_counts"])
         status_counts = facet_counts["status"]
         empty_status_count = status_counts[None]
         self.assertEquals(2, empty_status_count["count"])
@@ -280,14 +250,14 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
 
     def test_can_return_empty_facets_result_for_wiki_pages(self):
         #arrange
-        self.insert_wiki("W1","Some text")
+        self.insert_wiki("W1", "Some text")
         #act
         self.req.args[RequestParameters.TYPE] = "wiki"
         self.req.args[RequestParameters.QUERY] = "*"
         data = self.process_request()
         #assert
-        facet_counts =  data["facet_counts"]
-        self.assertEquals({}, facet_counts)
+        facet_counts = data["facet_counts"]
+        self.assertEquals([], facet_counts)
 
     def test_can_accept_multiple_filter_query_parameters(self):
         #arrange
@@ -320,10 +290,10 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         self.req.args[RequestParameters.FILTER_QUERY] = ['component:"c1"']
         data = self.process_request()
         #assert
-        facet_counts = data["facet_counts"]
+        facet_counts = dict(data["facet_counts"])
 
         milestone_facet_count = facet_counts["milestone"]
-        print unquote(milestone_facet_count[None]["href"])
+        self.env.log.debug(unquote(milestone_facet_count[None]["href"]))
 
     def test_can_handle_multiple_same(self):
         #arrange
@@ -335,11 +305,11 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         self.req.args[RequestParameters.FILTER_QUERY] = ['component:"c1"']
         data = self.process_request()
         #assert
-        facet_counts = data["facet_counts"]
+        facet_counts = dict(data["facet_counts"])
 
         component_facet_count = facet_counts["component"]
         c1_href = component_facet_count["c1"]["href"]
-        print unquote(c1_href)
+        self.env.log.debug(unquote(c1_href))
         self.assertEquals(
             1,
             self._count_parameter_in_url(c1_href, "fq", 'component:"c1"'))
@@ -357,14 +327,23 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         data = self.process_request()
         #assert
         current_filter_queries = data["active_filter_queries"]
-        self.assertEquals(2, len(current_filter_queries))
+        self.assertEquals(3, len(current_filter_queries))
 
-        component_filter =  current_filter_queries[0]
+        type_filter = current_filter_queries[0]
+        self.assertEquals('Ticket', type_filter["label"])
+        self.assertNotIn("type=", type_filter["href"])
+        self.assertNotIn('fq=', unquote(type_filter["href"]))
+
+        component_filter = current_filter_queries[1]
         self.assertEquals('component:"c1"', component_filter["label"])
-        self.assertNotIn("fq=", component_filter["href"])
+        self.assertIn('type=ticket', component_filter["href"])
+        self.assertNotIn('fq=component:"c1"',
+                         unquote(component_filter["href"]))
+        self.assertIn('fq=status:"new"', unquote(component_filter["href"]))
 
-        status_filter =  current_filter_queries[1]
+        status_filter = current_filter_queries[2]
         self.assertEquals('status:"new"', status_filter["label"])
+        self.assertIn('type=ticket', status_filter["href"])
         self.assertIn('fq=component:"c1"', unquote(status_filter["href"]))
         self.assertNotIn('fq=status:"new"', unquote(status_filter["href"]))
 
@@ -401,7 +380,7 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         self.req.args[RequestParameters.QUERY] = "*"
         data = self.process_request()
         #assert
-        ticket_facet_href = data["facet_counts"]["type"]["ticket"]["href"]
+        ticket_facet_href = dict(data["facet_counts"])["type"]["ticket"]["href"]
         ticket_facet_href = unquote(ticket_facet_href)
         self.assertIn("type=ticket", ticket_facet_href)
         self.assertNotIn("fq=", ticket_facet_href)
@@ -587,8 +566,7 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         data = self.process_request()
         #assert
         extra_search_options = dict(data["extra_search_fields"])
-        self.assertEquals("id, time desc", extra_search_options["sort"])
-        #self.assertNotIn("sort=", active_sort["href"])
+        self.assertEqual("id, time desc", extra_search_options["sort"])
 
     def test_that_document_summary_contains_highlighted_search_terms(self):
         term = "searchterm"
@@ -606,7 +584,7 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         term = "search_term"
         self.insert_wiki(term, term)
 
-        self.req.args[RequestParameters.QUERY] = "id:%s" % term
+        self.req.args[RequestParameters.QUERY] = "name:%s" % term
         data = self.process_request()
 
         title = str(data["results"].items[0]["title"])
@@ -652,10 +630,8 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
 
         for row in data["results"].items:
             title = row["title"]
-            print id, title
             self.assertIn(id, str(title))
 
-
     def test_that_id_is_highlighted_in_title(self):
         self.insert_ticket("some summary")
         id = "1"
@@ -663,7 +639,6 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         data = self.process_request()
         row = data["results"].items[0]
         title = row["title"]
-        print id, title
         self.assertIn('<em>%s</em>' % id, str(title))
 
     def test_that_content_summary_is_trimmed(self):
@@ -676,13 +651,182 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
             self.assertLess(len(row['content']), 500)
             self.assertLess(len(row['hilited_content']), 500)
 
+    def test_compatibility_with_legacy_search(self):
+        self.env.config.set('bhsearch', 'enable_redirect', "True")
+        self.req.path_info = '/search'
+
+        self.assertRaises(RequestDone, self.process_request)
+        self.assertIn('/bhsearch', self.redirect_url)
+        self.assertEqual(self.redirect_permanent, True)
+
+        self.req.args['wiki'] = 'on'
+        self.assertRaises(RequestDone, self.process_request)
+        redirect_url = unquote_plus(self.redirect_url)
+        self.assertIn('/bhsearch', redirect_url)
+        self.assertIn('type=wiki', redirect_url)
+        self.assertEqual(self.redirect_permanent, True)
+
+        self.req.args['ticket'] = 'on'
+        self.assertRaises(RequestDone, self.process_request)
+        redirect_url = unquote_plus(self.redirect_url)
+        self.assertIn('fq=type:(ticket OR wiki)', redirect_url)
+        self.assertIn('/bhsearch', self.redirect_url)
+        self.assertEqual(self.redirect_permanent, True)
+
+        self.req.args['milestone'] = 'on'
+        self.assertRaises(RequestDone, self.process_request)
+        redirect_url = unquote_plus(self.redirect_url)
+        self.assertIn('fq=type:(ticket OR wiki OR milestone)', redirect_url)
+        self.assertIn('/bhsearch', self.redirect_url)
+        self.assertEqual(self.redirect_permanent, True)
+
+        self.req.args['changeset'] = 'on'
+        self.assertRaises(RequestDone, self.process_request)
+        redirect_url = unquote_plus(self.redirect_url)
+        self.assertIn(
+            'fq=type:(ticket OR wiki OR milestone OR changeset)', redirect_url)
+        self.assertIn('/bhsearch', self.redirect_url)
+        self.assertEqual(self.redirect_permanent, True)
+
+    def test_opensearch_integration(self):
+        # pylint: disable=unused-variable
+        self.req.path_info = '/bhsearch/opensearch'
+        bhsearch = BloodhoundSearchModule(self.env)
+
+        url, data, x = bhsearch.process_request(self.req)
+
+        self.assertEqual(url, 'opensearch.xml')
+
+    def test_returns_correct_handler(self):
+        bhsearch = BloodhoundSearchModule(self.env)
+        tracsearch = self.env[TracSearchModule]
+
+        class PathInfoSetter(object):
+            # pylint: disable=incomplete-protocol
+            def __setitem__(other, key, value):
+                if key == "PATH_INFO":
+                    self.req.path_info = value
+        self.req.environ = PathInfoSetter()
+
+        self.env.config.set('bhsearch', 'enable_redirect', "True")
+
+        self.req.path_info = '/search'
+        self.assertIs(bhsearch.pre_process_request(self.req, tracsearch),
+                      bhsearch)
+
+        self.req.path_info = '/bhsearch'
+        self.assertIs(bhsearch.pre_process_request(self.req, tracsearch),
+                      bhsearch)
+
+        self.env.config.set('bhsearch', 'enable_redirect', "False")
+        # With redirect disabled, handler should not be changed.
+        self.req.path_info = '/search'
+        self.assertIs(bhsearch.pre_process_request(self.req, None),
+                      None)
+
+        self.req.path_info = '/bhsearch'
+        self.assertIs(bhsearch.pre_process_request(self.req, None),
+                      None)
+
+    def test_that_correct_search_handle_is_selected_for_quick_search(self):
+        bhsearch = BloodhoundSearchModule(self.env)
+
+        def process_request(path, enable_redirect, is_default):
+            # pylint: disable=unused-variable
+            self.req.path_info = path
+            self.env.config.set('bhsearch', 'enable_redirect',
+                                str(enable_redirect))
+            self.env.config.set('bhsearch', 'is_default', str(is_default))
+            template, data, content_type = \
+                bhsearch.post_process_request(self.req, '', {}, '')
+            return data
+
+        data = process_request('/', enable_redirect=False, is_default=False)
+        self.assertIn('search_handler', data)
+        self.assertEqual(data['search_handler'], self.req.href.search())
+
+        data = process_request('/', enable_redirect=True, is_default=False)
+        self.assertIn('search_handler', data)
+        self.assertEqual(data['search_handler'], self.req.href.bhsearch())
+
+        data = process_request('/', enable_redirect=False, is_default=True)
+        self.assertIn('search_handler', data)
+        self.assertEqual(data['search_handler'], self.req.href.bhsearch())
+
+        data = process_request('/', enable_redirect=True, is_default=True)
+        self.assertIn('search_handler', data)
+        self.assertEqual(data['search_handler'], self.req.href.bhsearch())
+
+        for is_default in [False, True]:
+            data = process_request('/search',
+                                   enable_redirect=False,
+                                   is_default=is_default)
+            self.assertIn('search_handler', data)
+            self.assertEqual(data['search_handler'], self.req.href.search())
+
+        for is_default in [False, True]:
+            data = process_request('/search',
+                                   enable_redirect=True,
+                                   is_default=is_default)
+            self.assertIn('search_handler', data)
+            self.assertEqual(data['search_handler'], self.req.href.bhsearch())
+
+        for enable_redirect in [False, True]:
+            for is_default in [False, True]:
+                data = process_request('/bhsearch',
+                                       enable_redirect=enable_redirect,
+                                       is_default=is_default)
+                self.assertIn('search_handler', data)
+                self.assertEqual(data['search_handler'],
+                                 self.req.href.bhsearch())
+
+    def test_that_active_query_is_set(self):
+        #arrange
+        self.insert_ticket("Ticket 1", component="c1", status="new")
+        self.insert_ticket("Ticket 2", component="c1", status="new")
+        #act
+        self.req.args[RequestParameters.TYPE] = "ticket"
+        self.req.args[RequestParameters.QUERY] = "Ticket"
+        self.req.args[RequestParameters.FILTER_QUERY] = [
+            'component:"c1"',
+            'status:"new"']
+        data = self.process_request()
+        #assert
+        active_query = data["active_query"]
+        self.assertEqual(active_query["label"], '"Ticket"')
+        self.assertEqual(active_query["query"], "Ticket")
+        self.assertNotIn("?q=", unquote(active_query["href"]))
+        self.assertNotIn("&q=", unquote(active_query["href"]))
+        self.assertIn("fq=", unquote(active_query["href"]))
+
+    def test_redirects_if_product_env_is_used_to_access_search(self):
+        self.env.config.set('bhsearch', 'global_quicksearch', "True")
+
+        with self.product('xxx'):
+            self.assertRaises(RequestDone, self.process_request)
+
+        self.assertIn('/bhsearch', self.redirect_url)
+        self.assertNotIn('/products', self.redirect_url)
+        self.assertNotIn('product_prefix=xxx', self.redirect_url)
+        self.assertTrue(self.redirect_permanent)
+
+    def test_adds_product_filter_when_global_quicksearch_is_disabled(self):
+        self.env.config.set('bhsearch', 'global_quicksearch', "false")
+
+        with self.product('xxx'):
+            self.assertRaises(RequestDone, self.process_request)
+
+        self.assertIn('/bhsearch', self.redirect_url)
+        self.assertNotIn('/products', self.redirect_url)
+        self.assertIn('product_prefix=xxx', self.redirect_url)
+        self.assertTrue(self.redirect_permanent)
+
     def _find_header(self, headers, name):
         for header in headers:
             if header["name"] == name:
                 return header
         raise Exception("Header not found: %s" % name)
 
-
     def _count_parameter_in_url(self, url, parameter_name, value):
         parameter_to_find = (parameter_name, value)
         parsed_parameters = parse_arg_list(url)
@@ -707,6 +851,7 @@ class WebUiTestCaseWithWhoosh(BaseBloodhoundSearchTest):
         for i in range(1, n+1):
             self.insert_wiki("test %s" % i)
 
+
 class RequestParametersTest(unittest.TestCase):
     def setUp(self):
         self.req = Mock(
@@ -747,7 +892,6 @@ class RequestParametersTest(unittest.TestCase):
         href = RequestParameters(self.req).create_href(
             sort=SortInstruction("field1", ASC))
         href = unquote(href)
-        print href
         self.assertIn("sort=field1+asc", href)
 
     def test_can_create_href_with_multiple_sort(self):
@@ -757,7 +901,6 @@ class RequestParametersTest(unittest.TestCase):
                 SortInstruction("field2", DESC),
             ])
         href = unquote(href)
-        print href
         self.assertIn("sort=field1+asc,+field2+desc", href)
 
     def _evaluate_sort(self, sort_condition):
