@@ -18,10 +18,15 @@
 #  specific language governing permissions and limitations
 #  under the License.
 from datetime import datetime
-
-import unittest
-import tempfile
+import os
 import shutil
+import sys
+import tempfile
+if sys.version_info[:2] < (2, 7):
+    import unittest2 as unittest
+else:
+    import unittest
+
 from bhsearch.api import ASC, DESC, SCORE, SortInstruction
 from bhsearch.query_parser import DefaultQueryParser
 from bhsearch.tests.base import BaseBloodhoundSearchTest
@@ -41,10 +46,6 @@ class WhooshBackendTestCase(BaseBloodhoundSearchTest):
         self.whoosh_backend.recreate_index()
         self.parser = DefaultQueryParser(self.env)
 
-    def tearDown(self):
-        shutil.rmtree(self.env.path)
-        self.env.reset_db()
-
     def test_can_retrieve_docs(self):
         self.whoosh_backend.add_doc(dict(id="1", type="ticket"))
         self.whoosh_backend.add_doc(dict(id="2", type="ticket"))
@@ -56,11 +57,11 @@ class WhooshBackendTestCase(BaseBloodhoundSearchTest):
         self.assertEqual(2, result.hits)
         docs = result.docs
         self.assertEqual(
-            {'id': '1', 'type': 'ticket', 'unique_id': 'ticket:1',
+            {'id': u'1', 'type': u'ticket', 'unique_id': u'empty:ticket:1',
              'score': u'1'},
             docs[0])
         self.assertEqual(
-            {'id': '2', 'type': 'ticket', 'unique_id': 'ticket:2',
+            {'id': u'2', 'type': u'ticket', 'unique_id': u'empty:ticket:2',
              'score': u'2'},
             docs[1])
 
@@ -70,7 +71,7 @@ class WhooshBackendTestCase(BaseBloodhoundSearchTest):
         self.print_result(result)
         docs = result.docs
         self.assertEqual(
-            {'id': '1', 'type': 'ticket', 'unique_id': 'ticket:1',
+            {'id': u'1', 'type': u'ticket', 'unique_id': u'empty:ticket:1',
                 "score": 1.0},
             docs[0])
 
@@ -84,7 +85,6 @@ class WhooshBackendTestCase(BaseBloodhoundSearchTest):
             {'id': '1', 'type': 'ticket'},
             docs[0])
 
-
     def test_can_survive_after_restart(self):
         self.whoosh_backend.add_doc(dict(id="1", type="ticket"))
         whoosh_backend2 = WhooshBackend(self.env)
@@ -416,19 +416,33 @@ class WhooshBackendTestCase(BaseBloodhoundSearchTest):
     def _highlighted(self, term):
         return '<em>%s</em>' % term
 
+
+class WhooshIndexCreationTests(BaseBloodhoundSearchTest):
+    def setUp(self):
+        super(WhooshIndexCreationTests, self).setUp()
+        self.index_dir = os.path.join(self.env.path, 'whoosh_index')
+        if not os.path.exists(self.index_dir):
+            os.mkdir(self.index_dir)
+
+    def test_does_not_automatically_create_index(self):
+        whoosh_backend = WhooshBackend(self.env)
+
+        self.assertIs(whoosh_backend.index, None)
+        self.assertEqual(whoosh_backend.is_index_outdated(), True)
+
+        whoosh_backend.recreate_index()
+        self.assertEqual(whoosh_backend.is_index_outdated(), False)
+        self.assertIsNot(whoosh_backend.index, None)
+
     def test_detects_that_index_needs_upgrade(self):
-        index_dir = self.whoosh_backend.index.storage.folder
         wrong_schema = Schema(content=TEXT())
-        ix = index.create_in(index_dir, schema=wrong_schema)
+        index.create_in(self.index_dir, schema=wrong_schema)
 
-        self.assertEqual(self.whoosh_backend.is_index_outdated(), False)
+        whoosh_backend = WhooshBackend(self.env)
+        self.assertEqual(whoosh_backend.is_index_outdated(), True)
 
-        # Inform WhooshBackend about the new index
-        self.whoosh_backend.index = ix
-        self.assertEqual(self.whoosh_backend.is_index_outdated(), True)
-        # Recreate index
-        self.whoosh_backend.recreate_index()
-        self.assertEqual(self.whoosh_backend.is_index_outdated(), False)
+        whoosh_backend.recreate_index()
+        self.assertEqual(whoosh_backend.is_index_outdated(), False)
 
 
 class WhooshFunctionalityTestCase(unittest.TestCase):
@@ -461,44 +475,10 @@ class WhooshFunctionalityTestCase(unittest.TestCase):
                 maptype=sorting.Count,
             )
             facets = self._load_facets(r)
-            print len(r) == 2
-        print facets
         self.assertEquals(
             {'status': {None: 1, 'New': 1}, 'type': {'type1': 1, 'type2': 1}},
             facets)
 
-    def test_can_use_query_and_groupedby_empty_field(self):
-        """
-        Whoosh 2.4 raises an error when simultaneously using filters and facets
-        in search:
-            AttributeError: 'FacetCollector' object has no attribute 'offset'
-
-        The problem should be fixed in the next release. For more info read
-        https://bitbucket.org/mchaput/whoosh/issue/274
-
-        For the time of being, whoosh-backend have to introduce workaround in
-        order to fix the problem. This unit-test is just a reminder to remove
-        workaround when the fixed version of Whoosh is applied.
-        """
-        schema = Schema(
-                unique_id=ID(stored=True, unique=True),
-                type=ID(stored=True),
-                )
-
-        ix = index.create_in(self.index_dir, schema=schema)
-        with ix.writer() as w:
-            w.add_document(unique_id=u"1", type=u"type1")
-            w.add_document(unique_id=u"2", type=u"type2")
-
-        with ix.searcher() as s:
-            with self.assertRaises(AttributeError):
-                s.search(
-                    query.Every(),
-                    groupedby=("type"),
-                    maptype=sorting.Count,
-                    filter=query.Term("type", "type1")
-                )
-
     def test_out_of_range_on_empty_facets(self):
         """
         Whoosh raises exception IndexError: list index out of range
@@ -606,6 +586,24 @@ class WhooshFunctionalityTestCase(unittest.TestCase):
                              u'(content:stop OR id:stop))')
             self.assertEqual(len(s.search(q)), 1)
 
+    def test_can_filter_to_no_results(self):
+        schema = Schema(
+            id=ID(stored=True),
+            filter=TEXT(stored=True),
+        )
+
+        ix = index.create_in(self.index_dir, schema=schema)
+        with ix.writer() as w:
+            w.add_document(id=u"1", filter=u"f1")
+            w.add_document(id=u"2", filter=u"f2")
+
+        with ix.searcher() as s:
+            r = s.search(
+                query.Every(),
+                filter=QueryParser('', schema).parse(u"filter:other")
+            )
+        self.assertEquals(len(r), 0)
+
 
 class WhooshEmptyFacetErrorWorkaroundTestCase(BaseBloodhoundSearchTest):
     def setUp(self):
@@ -637,7 +635,6 @@ class WhooshEmptyFacetErrorWorkaroundTestCase(BaseBloodhoundSearchTest):
             query_parameters)
 
         result_filter = query_parameters["filter"]
-        print result_filter
         self.assertEquals('(type:ticket AND milestone:empty)',
             str(result_filter))
 
@@ -649,8 +646,8 @@ class WhooshEmptyFacetErrorWorkaroundTestCase(BaseBloodhoundSearchTest):
             query_parameters)
 
         result_filter = query_parameters["filter"]
-        print result_filter
-        self.assertEquals('(type:ticket AND milestone:aaa)', str(result_filter))
+        self.assertEquals('(type:ticket AND milestone:aaa)',
+            str(result_filter))
 
 def suite():
     test_suite = unittest.TestSuite()
