@@ -23,20 +23,19 @@ from genshi.core import TEXT
 from genshi.filters.transform import Transformer
 from genshi.output import DocType
 
-from trac.config import Option, BoolOption
-from trac.core import *
-from trac.config import ListOption
+from trac.config import ListOption, Option
+from trac.core import Component, TracError, implements
 from trac.mimeview.api import get_mimetype
 from trac.resource import Resource
-from trac.ticket.api import TicketSystem
-from trac.ticket.model import Ticket
+from trac.ticket.model import Ticket, Milestone
 from trac.ticket.notification import TicketNotifyEmail
 from trac.ticket.web_ui import TicketModule
 from trac.util.compat import set
+from trac.util.presentation import to_json
 from trac.util.translation import _
 from trac.versioncontrol.web_ui.browser import BrowserModule
 from trac.web.api import IRequestFilter, IRequestHandler, ITemplateStreamFilter
-from trac.web.chrome import (add_script, add_stylesheet, INavigationContributor,
+from trac.web.chrome import (add_stylesheet, INavigationContributor,
                              ITemplateProvider, prevnext_nav, Chrome)
 from trac.wiki.admin import WikiAdmin
 
@@ -46,15 +45,15 @@ from bhdashboard.util import dummy_request
 from bhdashboard.web_ui import DashboardModule
 from bhdashboard import wiki
 
-from pkg_resources import get_distribution
-from urlparse import urlparse
-from wsgiref.util import setup_testing_defaults
+from multiproduct.env import ProductEnvironment
+from multiproduct.web_ui import PRODUCT_RE, ProductModule
 
 try:
     from multiproduct.ticket.web_ui import ProductTicketModule
 except ImportError:
     ProductTicketModule = None
 
+
 class BloodhoundTheme(ThemeBase):
     """Look and feel of Bloodhound issue tracker.
     """
@@ -67,74 +66,90 @@ class BloodhoundTheme(ThemeBase):
     )
     BLOODHOUND_TEMPLATE_MAP = {
         # Admin
-        'admin_basics.html' : ('bh_admin_basics.html', None),
-        'admin_components.html' : ('bh_admin_components.html', None),
-        'admin_enums.html' : ('bh_admin_enums.html', None),
-        'admin_logging.html' : ('bh_admin_logging.html', None),
-        'admin_milestones.html' : ('bh_admin_milestones.html', None),
-        'admin_perms.html' : ('bh_admin_perms.html', None),
-        'admin_plugins.html' : ('bh_admin_plugins.html', None),
-        'admin_repositories.html' : ('bh_admin_repositories.html', None),
-        'admin_versions.html' : ('bh_admin_versions.html', None),
-        'admin_products.html' : ('bh_admin_products.html', None),
+        'admin_accountsconfig.html': ('bh_admin_accountsconfig.html', '_modify_admin_breadcrumb'),
+        'admin_accountsnotification.html': ('bh_admin_accountsnotification.html', '_modify_admin_breadcrumb'),
+        'admin_basics.html': ('bh_admin_basics.html', '_modify_admin_breadcrumb'),
+        'admin_components.html': ('bh_admin_components.html', '_modify_admin_breadcrumb'),
+        'admin_enums.html': ('bh_admin_enums.html', '_modify_admin_breadcrumb'),
+        'admin_logging.html': ('bh_admin_logging.html', '_modify_admin_breadcrumb'),
+        'admin_milestones.html': ('bh_admin_milestones.html', '_modify_admin_breadcrumb'),
+        'admin_perms.html': ('bh_admin_perms.html', '_modify_admin_breadcrumb'),
+        'admin_plugins.html': ('bh_admin_plugins.html', '_modify_admin_breadcrumb'),
+        'admin_products.html': ('bh_admin_products.html', '_modify_admin_breadcrumb'),
+        'admin_repositories.html': ('bh_admin_repositories.html', '_modify_admin_breadcrumb'),
+        'admin_users.html': ('bh_admin_users.html', '_modify_admin_breadcrumb'),
+        'admin_versions.html': ('bh_admin_versions.html', '_modify_admin_breadcrumb'),
+
+        # no template substitutions below - use the default template,
+        # but call the modifier nonetheless
+        'repository_links.html': ('repository_links.html', '_modify_admin_breadcrumb'),
 
         # Preferences
-        'prefs.html' : ('bh_prefs.html', None),
-        'prefs_advanced.html' : ('bh_prefs_advanced.html', None),
-        'prefs_datetime.html' : ('bh_prefs_datetime.html', None),
-        'prefs_general.html' : ('bh_prefs_general.html', None),
-        'prefs_language.html' : ('bh_prefs_language.html', None),
-        'prefs_keybindings.html' : ('bh_prefs_keybindings.html', None),
-        'prefs_pygments.html' : ('bh_prefs_pygments.html', None),
-        'prefs_userinterface.html' : ('bh_prefs_userinterface.html', None),
+        'prefs.html': ('bh_prefs.html', None),
+        'prefs_account.html': ('bh_prefs_account.html', None),
+        'prefs_advanced.html': ('bh_prefs_advanced.html', None),
+        'prefs_datetime.html': ('bh_prefs_datetime.html', None),
+        'prefs_general.html': ('bh_prefs_general.html', None),
+        'prefs_keybindings.html': ('bh_prefs_keybindings.html', None),
+        'prefs_language.html': ('bh_prefs_language.html', None),
+        'prefs_pygments.html': ('bh_prefs_pygments.html', None),
+        'prefs_userinterface.html': ('bh_prefs_userinterface.html', None),
 
         # Search
-        'search.html' : ('bh_search.html', '_modify_search_data'),
+        'search.html': ('bh_search.html', '_modify_search_data'),
 
         # Wiki
-        'wiki_delete.html' : ('bh_wiki_delete.html', None),
-        'wiki_diff.html' : ('bh_wiki_diff.html', None),
-        'wiki_edit.html' : ('bh_wiki_edit.html', None),
-        'wiki_rename.html' : ('bh_wiki_rename.html', None),
-        'wiki_view.html' : ('bh_wiki_view.html', '_modify_wiki_page_path'),
+        'wiki_delete.html': ('bh_wiki_delete.html', None),
+        'wiki_diff.html': ('bh_wiki_diff.html', None),
+        'wiki_edit.html': ('bh_wiki_edit.html', None),
+        'wiki_rename.html': ('bh_wiki_rename.html', None),
+        'wiki_view.html': ('bh_wiki_view.html', '_modify_wiki_page_path'),
 
         # Ticket
-        'milestone_edit.html' : ('bh_milestone_edit.html', None),
-        'milestone_delete.html' : ('bh_milestone_delete.html', None),
-        'milestone_view.html' : ('bh_milestone_view.html', '_modify_roadmap_css'),
-        'query.html' : ('bh_query.html', None),
-        'report_delete.html' : ('bh_report_delete.html', None),
-        'report_edit.html' : ('bh_report_edit.html', None), 
-        'report_list.html' : ('bh_report_list.html', None),
-        'report_view.html' : ('bh_report_view.html', None),
-        'ticket.html' : ('bh_ticket.html', '_modify_ticket'),
-        'ticket_preview.html' : ('bh_ticket_preview.html', None),
-        'ticket_delete.html' : ('bh_ticket_delete.html', None),
+        'diff_view.html': ('bh_diff_view.html', None),
+        'manage.html': ('manage.html', '_modify_resource_breadcrumb'),
+        'milestone_edit.html': ('bh_milestone_edit.html', '_modify_roadmap_page'),
+        'milestone_delete.html': ('bh_milestone_delete.html', '_modify_roadmap_page'),
+        'milestone_view.html': ('bh_milestone_view.html', '_modify_roadmap_page'),
+        'query.html': ('bh_query.html', '_add_products_general_breadcrumb'),
+        'report_delete.html': ('bh_report_delete.html', '_add_products_general_breadcrumb'),
+        'report_edit.html': ('bh_report_edit.html', '_add_products_general_breadcrumb'),
+        'report_list.html': ('bh_report_list.html', '_add_products_general_breadcrumb'),
+        'report_view.html': ('bh_report_view.html', '_add_products_general_breadcrumb'),
+        'roadmap.html': ('roadmap.html', '_modify_roadmap_page'),
+        'ticket.html': ('bh_ticket.html', '_modify_ticket'),
+        'ticket_delete.html': ('bh_ticket_delete.html', None),
+        'ticket_preview.html': ('bh_ticket_preview.html', None),
 
         # Attachment
-        'attachment.html' : ('bh_attachment.html', None),
-        'preview_file.html' : ('bh_preview_file.html', None),
+        'attachment.html': ('bh_attachment.html', None),
+        'preview_file.html': ('bh_preview_file.html', None),
 
         # Version control
-        'revisionlog.html' : ('bh_revisionlog.html', '_modify_browser'),
-        'browser.html' : ('bh_browser.html', '_modify_browser'),
-        'dir_entries.html' : ('bh_dir_entries.html', None),
+        'browser.html': ('bh_browser.html', '_modify_browser'),
+        'dir_entries.html': ('bh_dir_entries.html', None),
+        'revisionlog.html': ('bh_revisionlog.html', '_modify_browser'),
 
         # Multi Product
-        'product_view.html' : ('bh_product_view.html', None),
+        'product_view.html': ('bh_product_view.html', '_add_products_general_breadcrumb'),
 
         # General purpose
-        'about.html' : ('bh_about.html', None),
-        'history_view.html' : ('bh_history_view.html', None),
-        'timeline.html' : ('bh_timeline.html', None),
+        'about.html': ('bh_about.html', None),
+        'history_view.html': ('bh_history_view.html', None),
+        'timeline.html': ('bh_timeline.html', None),
 
         # Account manager plugin
-        'login.html' : ('bh_login.html', None),
+        'account_details.html': ('bh_account_details.html', None),
+        'login.html': ('bh_login.html', None),
+        'register.html': ('bh_register.html', None),
+        'reset_password.html': ('bh_reset_password.html', None),
+        'user_table.html': ('bh_user_table.html', None),
+        'verify_email.html': ('bh_verify_email.html', None),
     }
     BOOTSTRAP_CSS_DEFAULTS = (
         # ('XPath expression', ['default', 'bootstrap', 'css', 'classes'])
-        ("body//table[not(contains(@class, 'table'))]", # TODO: Accurate ?
-                ['table', 'table-condensed']),
+        ("body//table[not(contains(@class, 'table'))]",  # TODO: Accurate ?
+         ['table', 'table-condensed']),
     )
     
     labels_application_short = Option('labels', 'application_short', 
@@ -154,23 +169,25 @@ class BloodhoundTheme(ThemeBase):
     labels_footer_right = Option('labels', 'footer_right', '',
         """Text to use as the right aligned footer""")
 
-    is_bhsearch_default = BoolOption('bhsearch', 'is_default', False)
-
     _wiki_pages = None
     Chrome.default_html_doctype = DocType.HTML5
 
     implements(IRequestFilter, INavigationContributor, ITemplateProvider,
                ITemplateStreamFilter)
 
+    from trac.web import main
+    main.default_tracker = 'http://issues.apache.org/bloodhound'
+
     def _get_whitelabelling(self):
         """Gets the whitelabelling config values"""
-        return dict(
-            application_short = self.labels_application_short,
-            application_full = self.labels_application_full,
-            footer_left_prefix = self.labels_footer_left_prefix,
-            footer_left_postfix = self.labels_footer_left_postfix,
-            footer_right = self.labels_footer_right,
-            application_version = application_version)
+        return {
+            'application_short': self.labels_application_short,
+            'application_full': self.labels_application_full,
+            'footer_left_prefix': self.labels_footer_left_prefix,
+            'footer_left_postfix': self.labels_footer_left_postfix,
+            'footer_right': self.labels_footer_right,
+            'application_version': application_version
+        }
 
     # ITemplateStreamFilter methods
 
@@ -195,9 +212,9 @@ class BloodhoundTheme(ThemeBase):
             return attr_modifier
         
         # Insert default bootstrap CSS classes if necessary
-        for xpath, classes in self.BOOTSTRAP_CSS_DEFAULTS :
+        for xpath, classes in self.BOOTSTRAP_CSS_DEFAULTS:
             tx = tx.end().select(xpath) \
-                    .attr('class', add_classes(classes))
+                .attr('class', add_classes(classes))
          
         # Rename wiki guide links
         tx = tx.end() \
@@ -239,7 +256,8 @@ class BloodhoundTheme(ThemeBase):
         """Post process request filter.
         Removes all trac provided css if required"""
         
-        if template is None and data is None and sys.exc_info() == (None, None, None):
+        if template is None and data is None and \
+                sys.exc_info() == (None, None, None):
             return template, data, content_type
         
         def is_active_theme():
@@ -252,7 +270,11 @@ class BloodhoundTheme(ThemeBase):
 
         req.chrome['labels'] = self._get_whitelabelling()
 
-        links = req.chrome.get('links',{})
+        if data is not None:
+            data['product_list'] = \
+                ProductModule.get_product_list(self.env, req)
+
+        links = req.chrome.get('links', {})
         # replace favicon if appropriate
         if self.env.project_icon == 'common/trac.ico':
             bh_icon = 'theme/img/bh.ico'
@@ -266,23 +288,25 @@ class BloodhoundTheme(ThemeBase):
         is_active_theme = is_active_theme()
         if self.disable_all_trac_css and is_active_theme:
             if self.disable_all_trac_css:
-                stylesheets = links.get('stylesheet',[])
+                stylesheets = links.get('stylesheet', [])
                 if stylesheets:
-                    path = req.base_path + '/chrome/common/css/'
+                    path = '/chrome/common/css/'
                     _iter = ([ss, ss.get('href', '')] for ss in stylesheets)
-                    links['stylesheet'] = [ss for ss, href in _iter 
-                            if not href.startswith(path) or
-                            href.rsplit('/', 1)[-1] in self.BLOODHOUND_KEEP_CSS]
-            template, modifier = self.BLOODHOUND_TEMPLATE_MAP.get(
-                    template, (template, None))
+                    links['stylesheet'] = \
+                        [ss for ss, href in _iter if not path in href or
+                         href.rsplit('/', 1)[-1] in self.BLOODHOUND_KEEP_CSS]
+            template, modifier = \
+                self.BLOODHOUND_TEMPLATE_MAP.get(template, (template, None))
             if modifier is not None:
                 modifier = getattr(self, modifier)
                 modifier(req, template, data, content_type, is_active_theme)
 
         if is_active_theme and data is not None:
-            data['responsive_layout'] = self.env.config.getbool(
-                    'bloodhound', 'responsive_layout', 'true')
-            data['is_bhsearch_default'] = self.is_bhsearch_default
+            data['responsive_layout'] = \
+                self.env.config.getbool('bloodhound', 'responsive_layout',
+                                        'true')
+            data['bhrelations'] = \
+                self.env.config.getbool('components', 'bhrelations.*', 'false')
 
         return template, data, content_type
 
@@ -310,21 +334,27 @@ class BloodhoundTheme(ThemeBase):
         if is_active:
             # Insert query string in search box (see bloodhound_theme.html)
             req.search_query = data.get('query')
-            # Breadcrumbs nav
-            data['resourcepath_template'] = 'bh_path_search.html'
             # Context nav
             prevnext_nav(req, _('Previous'), _('Next'))
+        # Breadcrumbs nav
+        data['resourcepath_template'] = 'bh_path_search.html'
 
-    def _modify_wiki_page_path(self, req, template, data, content_type, is_active):
+    def _modify_wiki_page_path(self, req, template, data, content_type,
+                               is_active):
         """Override wiki breadcrumbs nav items
         """
         if is_active:
             data['resourcepath_template'] = 'bh_path_wikipage.html'
 
-    def _modify_roadmap_css(self, req, template, data, content_type, is_active):
-        """Insert roadmap.css
+    def _modify_roadmap_page(self, req, template, data, content_type,
+                             is_active):
+        """Insert roadmap.css + products breadcrumb
         """
         add_stylesheet(req, 'dashboard/css/roadmap.css')
+        self._add_products_general_breadcrumb(req, template, data,
+                                              content_type, is_active)
+        data['milestone_list'] = [m.name for m in Milestone.select(self.env)]
+        req.chrome['ctxtnav'] = []
 
     def _modify_ticket(self, req, template, data, content_type, is_active):
         """Ticket modifications
@@ -332,6 +362,21 @@ class BloodhoundTheme(ThemeBase):
         self._modify_resource_breadcrumb(req, template, data, content_type,
                                          is_active)
 
+        #add a creation event to the changelog if the ticket exists
+        if data['ticket'].exists:
+            data['changes'] = [{'comment': '',
+                                'author': data['author_id'],
+                                'fields': {u'reported': {'label': u'Reported'},
+                                           },
+                                'permanent': 1,
+                                'cnum': 0,
+                                'date': data['start_time'],
+                                },
+                               ] + data['changes']
+        #and set default order
+        if not req.session.get('ticket_comments_order'):
+            req.session['ticket_comments_order'] = 'newest'
+
     def _modify_resource_breadcrumb(self, req, template, data, content_type,
                                     is_active):
         """Provides logic for breadcrumb resource permissions
@@ -344,16 +389,43 @@ class BloodhoundTheme(ThemeBase):
                 res = Resource(resname, data['ticket'][resname])
                 data['path_show_' + resname] = permname in req.perm(res)
 
+            # add milestone list + current milestone to the breadcrumb
+            data['milestone_list'] = [m.name
+                                      for m in Milestone.select(self.env)]
+            mname = data['ticket']['milestone']
+            if mname:
+                data['milestone'] = Milestone(self.env, mname)
+
+    def _modify_admin_breadcrumb(self, req, template, data, content_type, is_active):
+        # override 'normal' product list with the admin one
+        glsettings = (None, _('(Global settings)'), req.href.admin())
+        admin_url = lambda x: req.href.products(x, 'admin')
+        data['admin_product_list'] = [glsettings] + \
+            ProductModule.get_product_list(self.env, req, admin_url)
+
+        if isinstance(req.perm.env, ProductEnvironment):
+            product = req.perm.env.product
+            data['admin_current_product'] = \
+                (product.prefix, product.name,
+                 req.href.products(product.prefix, 'admin'))
+        else:
+            data['admin_current_product'] = glsettings
+        data['resourcepath_template'] = 'bh_path_general.html'
+
     def _modify_browser(self, req, template, data, content_type, is_active):
         """Locate path to file in breadcrumbs area rather than title.
         Add browser-specific CSS.
         """
-        data.update(dict(
-                resourcepath_template='bh_path_links.html',
-                path_depth_limit=2
-            ))
+        data.update({
+            'resourcepath_template': 'bh_path_links.html',
+            'path_depth_limit': 2
+        })
         add_stylesheet(req, 'theme/css/browser.css')
 
+    def _add_products_general_breadcrumb(self, req, template, data,
+                                         content_type, is_active):
+        data['resourcepath_template'] = 'bh_path_general.html'
+
     # INavigationContributor methods
 
     def get_active_navigation_item(self, req):
@@ -367,11 +439,12 @@ class BloodhoundTheme(ThemeBase):
                        tag.a(_('Browse Source'),
                              href=req.href.wiki('TracRepositoryAdmin')))
 
+
 class QuickCreateTicketDialog(Component):
     implements(IRequestFilter, IRequestHandler)
 
     qct_fields = ListOption('ticket', 'quick_create_fields', 
-                            'product,version,type',
+                            'product, version, type',
         doc="""Multiple selection fields displayed in create ticket menu""")
 
     # IRequestFilter(Interface):
@@ -390,17 +463,37 @@ class QuickCreateTicketDialog(Component):
             # no ticket module so no create ticket button
             return template, data, content_type
 
-        if (template, data, content_type) != (None,) * 3: # TODO: Check !
+        if (template, data, content_type) != (None,) * 3:  # TODO: Check !
             if data is None:
                 data = {}
-            fakereq = dummy_request(self.env)
+            req = dummy_request(self.env)
             ticket = Ticket(self.env)
-            tm._populate(fakereq, ticket, False)
-            all_fields = dict([f['name'], f] \
-                              for f in tm._prepare_fields(fakereq, ticket) \
+            tm._populate(req, ticket, False)
+            all_fields = dict([f['name'], f]
+                              for f in tm._prepare_fields(req, ticket)
                               if f['type'] == 'select')
-            data['qct'] = {'fields': [all_fields[k] for k in self.qct_fields
-                                      if k in all_fields]}
+
+            product_field = all_fields['product']
+            if product_field:
+                if self.env.product:
+                    product_field['value'] = self.env.product.prefix
+                else:
+                    # Global scope, now check default_product_prefix is valid
+                    default_prefix = self.config.get('multiproduct',
+                                                     'default_product_prefix')
+                    try:
+                        ProductEnvironment.lookup_env(self.env, default_prefix)
+                    except LookupError:
+                        product_field['value'] = product_field['options'][0]
+                    else:
+                        product_field['value'] = default_prefix
+
+            data['qct'] = {
+                'fields': [all_fields[k] for k in self.qct_fields
+                           if k in all_fields],
+                'hidden_fields': [all_fields[k] for k in all_fields.keys()
+                                  if k not in self.qct_fields]
+            }
         return template, data, content_type
 
     # IRequestHandler methods
@@ -408,7 +501,9 @@ class QuickCreateTicketDialog(Component):
     def match_request(self, req):
         """Handle requests sent to /qct
         """
-        return req.path_info == '/qct'
+        m = PRODUCT_RE.match(req.path_info)
+        return req.path_info == '/qct' or \
+            (m and m.group('pathinfo').strip('/') == 'qct')
 
     def process_request(self, req):
         """Forward new ticket request to `trac.ticket.web_ui.TicketModule`
@@ -419,14 +514,15 @@ class QuickCreateTicketDialog(Component):
             req.perm.require('TICKET_CREATE')
             summary = req.args.pop('field_summary', '')
             desc = ""
-            attrs = dict([k[6:], v] for k,v in req.args.iteritems() \
-                                    if k.startswith('field_'))
-            ticket_id = self.create(req, summary, desc, attrs, True)
+            attrs = dict([k[6:], v] for k, v in req.args.iteritems()
+                         if k.startswith('field_'))
+            product, tid = self.create(req, summary, desc, attrs, True)
         except Exception, exc:
             self.log.exception("BH: Quick create ticket failed %s" % (exc,))
             req.send(str(exc), 'plain/text', 500)
         else:
-            req.send(str(ticket_id), 'plain/text')
+            req.send(to_json({'product': product, 'id': tid}),
+                     'application/json')
 
     def _get_ticket_module(self):
         ptm = None
@@ -440,7 +536,7 @@ class QuickCreateTicketDialog(Component):
         return tm
 
     # Public API
-    def create(self, req, summary, description, attributes = {}, notify=False):
+    def create(self, req, summary, description, attributes={}, notify=False):
         """ Create a new ticket, returning the ticket ID. 
 
         PS: Borrowed from XmlRpcPlugin.
@@ -454,10 +550,7 @@ class QuickCreateTicketDialog(Component):
         t['status'] = 'new'
         t['resolution'] = ''
         t.insert()
-        # Call ticket change listeners
-        ts = TicketSystem(self.env)
-        for listener in ts.change_listeners:
-            listener.ticket_created(t)
+
         if notify:
             try:
                 tn = TicketNotifyEmail(self.env)
@@ -465,6 +558,7 @@ class QuickCreateTicketDialog(Component):
             except Exception, e:
                 self.log.exception("Failure sending notification on creation "
                                    "of ticket #%s: %s" % (t.id, e))
-        return t.id
+        return t['product'], t.id
 
+from pkg_resources import get_distribution
 application_version = get_distribution('BloodhoundTheme').version
