@@ -62,7 +62,8 @@ BASE_CONFIG = {'components': {'bhtheme.*': 'enabled',
                               'permredirect.*': 'enabled',
                               'themeengine.api.*': 'enabled',
                               'themeengine.web_ui.*': 'enabled',
-                              'bhsearch.*': 'disabled',
+                              'bhsearch.*': 'enabled',
+                              'bhrelations.*': 'enabled',
                               'trac.ticket.web_ui.ticketmodule': 'disabled',
                               'trac.ticket.report.reportmodule': 'disabled',
                               },
@@ -76,15 +77,42 @@ BASE_CONFIG = {'components': {'bhtheme.*': 'enabled',
                'theme': {'theme': 'bloodhound',},
                'trac': {'mainnav': ','.join(['dashboard', 'wiki', 'browser',
                                              'tickets', 'newticket', 'timeline',
-                                             'roadmap', 'search', 'admin']),},
-               'project': {'footer': ('Visit Apache Bloodhound at<br />'
-                                      '<a href="%(site)s">%(site)s</a>'
+                                             'roadmap', 'search', 'admin']),
+                        'environment_factory': '',
+                        'request_factory': '',},
+               'project': {'footer': ('Get involved with '
+                                      '<a href="%(site)s">Apache Bloodhound</a>'
                                       % {'site': BH_PROJECT_SITE,}),},
                'labels': {'application_short': 'Bloodhound',
                           'application_full': 'Apache Bloodhound',
                           'footer_left_prefix': '',
                           'footer_left_postfix': '',
                           'footer_right': ''},
+               'bhsearch': {'is_default': 'true', 'enable_redirect': 'true'},
+               'bhrelations': {
+                   'global_validators':
+                       'NoSelfReferenceValidator,ExclusiveValidator,'
+                       'BlockerValidator',
+               },
+               'bhrelations_links': {
+                    'children.label': 'Child',
+                    'dependency': 'dependson,dependent',
+                    'dependency.validators': 'NoCycles,SingleProduct',
+                    'dependson.blocks': 'true',
+                    'dependson.label': 'Depends on',
+                    'dependent.label': 'Dependent',
+                    'oneway': 'refersto',
+                    'parent_children': 'parent,children',
+                    'parent.exclusive': 'true',
+                    'parent_children.validators':
+                        'OneToMany,SingleProduct,NoCycles',
+                    'refersto.label': 'Refers to',
+                    'duplicate': 'duplicateof,duplicatedby',
+                    'duplicate.validators': 'ReferencesOlder',
+                    'duplicateof.label': 'Duplicate of',
+                    'duplicatedby.label': 'Duplicated by',
+               },
+
                }
 
 ACCOUNTS_CONFIG = {'account-manager': {'account_changes_notify_addresses' : '',
@@ -106,6 +134,7 @@ ACCOUNTS_CONFIG = {'account-manager': {'account_changes_notify_addresses' : '',
                                   'acct_mgr.api.accountmanager' : 'enabled',
                                   'acct_mgr.guard.accountguard' : 'enabled',
                                   'acct_mgr.htfile.htdigeststore' : 'enabled',
+                                  'acct_mgr.macros.*': 'enabled',
                                   'acct_mgr.web_ui.accountmodule' : 'enabled',
                                   'acct_mgr.web_ui.loginmodule' : 'enabled',
                                   'trac.web.auth.loginmodule' : 'disabled',
@@ -175,7 +204,7 @@ class BloodhoundSetup(object):
         new_env =  os.path.join(environments_path, options['project'])
         tracini = os.path.abspath(os.path.join(new_env, 'conf', 'trac.ini'))
         baseini = os.path.abspath(os.path.join(new_env, 'conf', 'base.ini'))
-        options['inherit'] = baseini
+        options['inherit'] = '"' + baseini + '"'
 
         options['db'] = self._generate_db_str(options)
         if 'repo_type' not in options or options['repo_type'] is None:
@@ -187,6 +216,12 @@ class BloodhoundSetup(object):
                   "repository-type and the repository-path options."
             return False
 
+        custom_prefix = 'default_product_prefix'
+        if custom_prefix in options and options[custom_prefix]:
+            default_product_prefix = options[custom_prefix]
+        else:
+            default_product_prefix = '@'
+
         digestfile = os.path.abspath(os.path.join(new_env,
                                                   options['digestfile']))
         realm =  options['realm']
@@ -214,7 +249,23 @@ class BloodhoundSetup(object):
             print ("Warning: Environment already exists at %s." % new_env)
             self.writeconfig(tracini, [{'inherit': {'file': baseini},},])
 
-        self.writeconfig(baseini, [BASE_CONFIG, accounts_config])
+        base_config = dict(BASE_CONFIG)
+        environment_factory_path = os.path.abspath(
+                                      os.path.normpath(
+                                          os.path.join(options['sourcedir'],
+                                                               'bloodhound_multiproduct/multiproduct/hooks.py')))
+        request_factory_path = os.path.abspath(
+                                   os.path.normpath(
+                                       os.path.join(options['sourcedir'],
+                                                            'bloodhound_multiproduct/multiproduct/hooks.py')))
+        base_config['trac']['environment_factory'] = environment_factory_path
+        base_config['trac']['request_factory'] = request_factory_path
+        if default_product_prefix != '@':
+            base_config['multiproduct'] = dict(
+                default_product_prefix=default_product_prefix
+            )
+
+        self.writeconfig(baseini, [base_config, accounts_config])
 
         if os.path.exists(digestfile):
             backupfile(digestfile)
@@ -231,15 +282,30 @@ class BloodhoundSetup(object):
         bloodhound.onecmd('upgrade')
         pages = []
         pages.append(pkg_resources.resource_filename('bhdashboard',
-                                                     'default-pages'))
+                                                 'default-pages'))
+        pages.append(pkg_resources.resource_filename('bhsearch',
+                                                 'default-pages'))
         bloodhound.onecmd('wiki load %s' % " ".join(pages))
 
         print "Running wiki upgrades"
         bloodhound.onecmd('wiki upgrade')
         
-        print "Running wiki bh upgrades"
+        print "Running wiki Bloodhound upgrades"
         bloodhound.onecmd('wiki bh-upgrade')
 
+        print "Loading default product wiki"
+        bloodhound.onecmd('product admin %s wiki load %s' %
+                          (default_product_prefix,
+                           " ".join(pages)))
+
+        print "Running default product wiki upgrades"
+        bloodhound.onecmd('product admin %s wiki upgrade' %
+                          default_product_prefix)
+
+        print "Running default product wiki Bloodhound upgrades"
+        bloodhound.onecmd('product admin %s wiki bh-upgrade' %
+                          default_product_prefix)
+
         print """
 You can now start Bloodhound by running:
 
@@ -297,6 +363,9 @@ def handle_options():
     # Base Trac Options
     parser.add_option('--project', dest='project',
                       help='Set the top project name', default='main')
+    parser.add_option('--source_directory', dest='sourcedir',
+                      help='Specify root source code directory',
+                      default=os.path.normpath(os.path.join(os.getcwd(), '../'))),
     parser.add_option('--environments_directory', dest='envsdir',
                       help='Set the directory to contain environments',
                       default=os.path.join('bloodhound', 'environments'))
@@ -338,6 +407,11 @@ def handle_options():
                       help='specify the repository type - ')
     parser.add_option('--repository-path', dest='repo_path',
                       help='specify the repository type')
+
+    # Multiproduct options
+    parser.add_option('--default-product-prefix', dest='default_product_prefix',
+                      help='Specify prefix for default product (defaults to @')
+
     (options, args) = parser.parse_args()
     if args:
         print "Unprocessed options/arguments: ", args
