@@ -815,7 +815,7 @@ class Chrome(Component):
     def populate_data(self, req, data):
         d = self._default_context_data.copy()
         d['trac'] = {
-            'version': pkg_resources.resource_string('trac', 'TRAC_VERSION'),
+            'version': pkg_resources.resource_string('trac', 'TRAC_VERSION').strip(),
             'homepage': 'http://trac.edgewall.org/', # FIXME: use setup data
         }
 
@@ -884,6 +884,7 @@ class Chrome(Component):
         d.update({
             'context': web_context(req) if req else None,
             'Resource': Resource,
+            'Neighborhood': Neighborhood,
             'url_of': get_rel_url,
             'abs_url_of': get_abs_url,
             'name_of': partial(get_resource_name, self.env),
